/**
 * EasyBeans
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: easybeans@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: LocalLauncher.java 1941 2007-10-14 17:57:14Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.osgi.plugins.launcher;

import java.util.Enumeration;
import java.util.Properties;

import org.ow2.easybeans.osgi.plugins.launcher.api.LauncherException;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Launcher that is used in the same JVM.
 * @author Florent BENOIT
 */
public class LocalLauncher extends AbsLauncher {

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(LocalLauncher.class);

    /**
     * Manager of the framework.
     */
    private FrameworkManager frameworkManager = null;


    /**
     * Build a launcher with the given configuration object.
     * @param configuration the given configuration
     */
    public LocalLauncher(final Configuration configuration) {
        super(configuration);
    }

    /**
     * Start the framework manager used by this launcher.
     * @throws LauncherException if there is a failure during the start.
     */
    public void start() throws LauncherException {

        // Set system properties
        Properties jvmProperties = getConfiguration().getSystemProperties();
        if (jvmProperties != null) {
            Enumeration<Object> enumeration = jvmProperties.keys();
            while (enumeration.hasMoreElements()) {
                String key = enumeration.nextElement().toString();
                String value = jvmProperties.get(key).toString();
                // property already exists ?
                String oldValue = System.getProperty(key);
                if (!value.equals(oldValue)) {
                    System.setProperty(key, value);
                    logger.info("Adding System property '" + key + "' with value '" + value + "'");
                }
            }
        }


        logger.info("Launching the Local framework...");
        try {
           this.frameworkManager = FrameworkManagerHelper.getFrameworkManager(getConfiguration());
        } catch (FrameworkManagerException e) {
            throw new LauncherException("Cannot get framework manager", e);
        }


        try {
            this.frameworkManager.start();
        } catch (FrameworkManagerException e) {
            throw new LauncherException("Cannot start the framework manager", e);
        }

    }

    /**
     * Stop the framework manager used by this launcher.
     * @throws LauncherException if there is a failure during the stop.
     */
    public void stop() throws LauncherException {
        try {
            this.frameworkManager.stop();
        } catch (FrameworkManagerException e) {
            throw new LauncherException("Cannot stop the framework manager", e);
        }
        // Waiting for shutdown
        logger.info("Waiting for shutdown");
        try {
            Thread.sleep(getConfiguration().getWaitAfterStop() * SECONDS);
        } catch (InterruptedException e) {
            throw new LauncherException("Cannot wait", e);
        }
    }


}
